/*
 * MAPS v1.0 3/8/97 by Frank Reid and Cerebrum Software.                    *
 * Determines total and available disk space on physical hard disks,        *
 * including network drives.  Bypasses removable media, including CDROM.    *
 * Routines derived from WWIV v4.24 source and the Snippets archives.       *
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <dos.h>

double freek(int dr);
double totalk(int dr);
int drive_type(int dr);
double get_disk_free(int drive);

#pragma warn -par

main(int argc, char *argv[])
{
    float           total;
    char            tc[21];
    int             i;

    printf("\nDrive Map v1.0 3/8/97 by Frank Reid and Cerebrum Software.\n");

    total = 0.0;
    printf("\nDrive       Size       Free");
    printf("\n---------------------------");
    for (i = 3; i < 26; i++)
        total += get_disk_free(i);
    printf("\n---------------------------");
    sprintf(tc, "%.1fM", total / 1024.0);
    printf("\nSpace available: %10s\n\n", tc);
    return 0;

}

double freek(int dr)
{
    float           d;
    struct dfree    df;

    getdfree(dr, &df);
    d = (float) df.df_avail;
    d *= ((float) df.df_bsec);
    d *= ((float) df.df_sclus);
    d /= 1024.0;
    if (df.df_sclus == 0xffff)
        d = -1.0;
    return (d);
}

double totalk(int dr)
{
    float           d;
    struct dfree    df;

    getdfree(dr, &df);
    d = (float) df.df_total;
    d *= ((float) df.df_bsec);
    d *= ((float) df.df_sclus);
    d /= 1024.0;
    if (df.df_sclus == 0xffff)
        d = -1.0;
    return (d);
}

int drive_type(int dr)
{
    union REGS      regs;

    regs.x.ax = 0x4409;
    regs.h.bl = (unsigned char) dr;
    int86(0x21, &regs, &regs);
    if (!regs.x.cflag) {
        if (regs.x.dx & 0x1000)
            return 1;
        else
            if (regs.x.dx == 0x0800)
            return 2;
    }
    return 0;
}

double get_disk_free(int drive)
{
    union REGS      r;
    struct SREGS    s;
    char            fc[21],
                    tc[21];
    float           fs,
                    ts;

    r.x.ax = 0x6900;
    r.h.bl = drive;
    segread(&s);
    intdosx(&r, &r, &s);
    if ((r.x.cflag) && (drive_type(drive) > 1))
        return 0;
    else {
        fs = freek(drive);
        if (fs > 0) {
            ts = totalk(drive);
            sprintf(fc, "%.1fM", fs / 1024.0);
            sprintf(tc, "%.1fM", ts / 1024.0);
            printf("\n%c:     %10s%10s", drive + '@', tc, fc);
        }
    }
    return fs;
}
